from django import template
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from .. import import_item
from ..sitemenu_settings import MENUCLASS
Menu = import_item(MENUCLASS)

register = template.Library()


def render_sitemenu(context, template='_menu', catalogue_root=None):
    if not catalogue_root:
        nodes = Menu.objects.filter(enabled=True)
    else:
        nodes = Menu.objects.filter(enabled=True, full_url__startswith=catalogue_root.full_url).exclude(pk=catalogue_root.pk)
        if hasattr(catalogue_root, 'q_filters') and catalogue_root.q_filters:
            nodes = nodes.filter(catalogue_root.q_filters)
    return render_to_string('sitemenu/%s.html' % template, {'nodes': nodes}, context_instance=context)
register.simple_tag(takes_context=True)(render_sitemenu)


def is_activemenu(context, menu, active_text=' class="active"'):
    if menu.is_active(context['request'].get_full_path()):
        return active_text
    else:
        return ''
register.simple_tag(takes_context=True)(is_activemenu)


def render_breadcrumbs(context, template='_breadcrumbs'):
    try:
        menu = context['menu']
    except KeyError:
        return ''
    breadcrumbs = menu.get_breadcrumbs()
    return render_to_string('sitemenu/%s.html' % template, {'breadcrumbs': breadcrumbs}, context_instance=context)
register.simple_tag(takes_context=True)(render_breadcrumbs)


@register.tag
def recurse_sitemenu(parser, token):
    """
    Based on MPTT recursetree tag

    Iterates over the nodes in the tree, and renders the contained block for each node.
    This tag will recursively render children into the template variable {{ children }}.
    Only one database query is required (children are cached for the whole tree)

    Usage:
        <ul>
            {% recurse_sitemenu nodes %}
                <li>
                    {{ node.title }}
                    {% if node.has_childs %}
                        <ul>
                            {{ children }}
                        </ul>
                    {% endif %}
                </li>
            {% endrecurse_sitemenu %}
        </ul>
    """
    bits = token.contents.split()
    if len(bits) != 2:
        raise template.TemplateSyntaxError(_('%s tag requires a queryset') % bits[0])

    queryset_var = bits[1]

    template_nodes = parser.parse(('endrecurse_sitemenu',))
    parser.delete_first_token()

    class RecurseSiteMenuNode(template.Node):
        def __init__(self, template_nodes, queryset_var):
            self.template_nodes = template_nodes
            self.queryset_var = queryset_var

        def _render_node(self, context, node):
            bits = []
            context.push()
            for child in node.get_childs():
                context['node'] = child
                bits.append(self._render_node(context, child))
            context['node'] = node
            context['children'] = mark_safe(u''.join(bits))
            rendered = self.template_nodes.render(context)
            context.pop()
            return rendered

        def render(self, context):
            queryset = context[self.queryset_var]
            try:
                roots = queryset[0].create_tree(queryset)
            except IndexError:
                return ''
            bits = [self._render_node(context, node) for node in roots]
            return ''.join(bits)

    return RecurseSiteMenuNode(template_nodes, queryset_var)



from django.conf import settings

PAGES = getattr(settings, 'SITEMENU_PAGES', (
    ('text', 'Text page', 'sitemenu.views.render_menupage'),
    ('redr', 'Redirect page', 'sitemenu.views.render_redirectpage'),
    ('indx', 'Index page', 'sitemenu.views.render_menupage'),
))

MENUCLASS = getattr(settings, 'SITEMENU_MENUCLASS', 'sitemenu.models.Menu')



from django.shortcuts import render_to_response
from django.template import RequestContext
from django.http import Http404, HttpResponseRedirect, HttpResponsePermanentRedirect
from django.core.urlresolvers import reverse
from .sitemenu_settings import MENUCLASS
from . import import_item

Menu = import_item(MENUCLASS)


def dispatcher(request, url):
    url_add = []
    if url == '':
        try:
            menu = Menu.objects.filter(enabled=True, page_type='indx')[0]
        except:
            menu = None
    else:

        if not url.endswith('/'):
            return HttpResponsePermanentRedirect(reverse('dispatcher', kwargs={'url': url + '/'}))
        try:
            menu = Menu.objects.filter(full_url=url, enabled=True)[0]
        except:
            menu = None

        if not menu:
            url_arr = url.split('/')[:-1]
            while url_arr:
                try:
                    menu = Menu.objects.get(enabled=True, full_url='/'.join(url_arr) + '/')
                    break
                except:
                    url_add.append(url_arr[-1])
                    url_arr = url_arr[:-1]

        if menu and menu.page_type == 'indx':
            return HttpResponsePermanentRedirect(reverse('dispatcher', kwargs={'url': ''}))

    if not menu:
        raise Http404

    if menu.redirect_url:
        return HttpResponseRedirect(menu.redirect_url)
    if menu.redirect_to_first_child:
        return HttpResponseRedirect(menu._default_manager.filter(parent=menu)[0].get_absolute_url())

    return menu.render(request, list(reversed(url_add)))


def render_menupage(request, menu, url_add):
    if url_add:
        raise Http404
    return render_to_response('sitemenu/simplepage.html', {
            'menu': menu,
            'url_add': url_add,
        }, context_instance=RequestContext(request))


def render_redirectpage(request, menu, url_add):
    if url_add:
        raise Http404
    return HttpResponseRedirect(menu.redirect_url)



from django.contrib import admin
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from .sitemenu_settings import MENUCLASS


class SiteMenuAdmin(admin.ModelAdmin):
    prepopulated_fields = {"url": ("title",)}
    list_display = ('title', 'enabled')

    class Media:
        css = {
            'screen': ('/static/admin/sitemenu/css/no-theme/jquery-ui-1.9.0.custom.min.css', '/static/admin/sitemenu/css/sitemenu.css',),
        }

    def __init__(self, *args, **kwargs):
        super(SiteMenuAdmin, self).__init__(*args, **kwargs)

        self.list_display = list(self.list_display)

        if 'indented_short_title' not in self.list_display:
            if self.list_display[0] == 'action_checkbox':
                self.list_display[1] = 'indented_short_title'
            else:
                self.list_display[0] = 'indented_short_title'
        self.list_display_links = ('indented_short_title',)

    def indented_short_title(self, item):
        title = item.title
        span = '<span class="result_list__ident_span"></span>' * item.level
        return mark_safe('%s%s<div class="drag_handle_container"><div class="drag_handle"></div></div>' % (span, title))
    indented_short_title.short_description = _('title')
    indented_short_title.allow_tags = True


if MENUCLASS == 'sitemenu.models.Menu':
    from .models import Menu
    admin.site.register(Menu, SiteMenuAdmin)



from django.utils import importlib


def import_item(path):
    try:
        render_import = path.split('.')
        render_module = importlib.import_module('.'.join(render_import[:-1]))
        render_function = render_import[-1]
    except ImportError, e:
        raise ImportError("Could not import function '%s': %s" % (path, e))
    try:
        func = getattr(render_module, render_function)
    except AttributeError, e:
        raise AttributeError("Could not find function '%s' in %s" % (render_function, render_module))
    return func



from django.conf.urls import patterns, url
from .views import dispatcher
from .admin_views import save_menu_position

urlpatterns = patterns('',
    url(r'^admin/sitemenu/save_menu_position/$', save_menu_position, name='save_menu_position'),
    url(r'^(?P<url>.*)$', dispatcher, name='dispatcher'),
)



from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.http import HttpResponse
from .sitemenu_settings import MENUCLASS
from . import import_item

Menu = import_item(MENUCLASS)

@staff_member_required
@csrf_exempt
def save_menu_position(request):
    ret = HttpResponse()
    try:
        element = Menu.objects.get(pk=request.POST['element_id'])
        target = Menu.objects.get(pk=request.POST['target_id'])
        level = int(request.POST['level'])
        top_half = True if int(request.POST['top_half']) == 1 else False
    except:
        messages.add_message(request, messages.ERROR, _('Unexpected error.'))
        return ret

    if target.level != level:
        same_level = False
    else:
        same_level = True

    position = ''
    if top_half:
        if same_level:
            real_target = target
            position = 'before'
        else:
            messages.add_message(request, messages.ERROR, _('Wrong position.'))
            return ret
    else:
        if same_level:
            real_target = target
            position = 'after'
        else:
            real_target = target

    if element.pk in real_target.get_parents_ids_list():
        messages.add_message(request, messages.WARNING, _('Impossible movement.'))
        return ret

    if same_level:
        items = Menu.objects.filter(parent=real_target.parent)
        a = 0
        for item in items:
            if item == element:
                continue
            if item == real_target and position == 'before':
                element.parent = item.parent
                element.sort = a
                a += 1
                element.save(skip_tree_update=True)
            item.sort = a
            a += 1
            item.save(skip_tree_update=True)
            if item == real_target and position == 'after':
                element.parent = item.parent
                element.sort = a
                a += 1
                element.save(skip_tree_update=True)
    else:
        items = Menu.objects.filter(parent=real_target.pk)
        element.parent = real_target
        element.sort = 0
        element.save(skip_tree_update=True)
        a = 1
        for item in items:
            if item == element:
                continue
            item.sort = a
            a += 1
            item.save(skip_tree_update=True)

    element.rebuild()

    messages.add_message(request, messages.SUCCESS, _('Item moved.'))

    return ret



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from .sitemenu_settings import PAGES as PAGES_TYPES, MENUCLASS
from . import import_item


class SiteMenu(models.Model):

    PAGES = PAGES_TYPES

    MAX_LEVELS = 5
    MAX_ITEMS = 99
    TYPE_TYPES = [(x[0], x[1]) for x in PAGES]

    # Tree fields
    sort       = models.IntegerField(_('sort'), default=0)
    sortorder  = models.CharField(max_length=MAX_LEVELS * len(str(MAX_ITEMS)), editable=False)
    level      = models.PositiveSmallIntegerField(editable=False, default=0)
    has_childs = models.BooleanField(default=False, editable=False)
    parent     = models.ForeignKey('self', null=True, blank=True, verbose_name=_('parent'))
    parents_list = models.CharField(max_length=MAX_LEVELS * 5, editable=False, null=True, blank=True)

    # Menu field
    full_url       = models.CharField(max_length=255, null=True, blank=True, editable=False, unique=True)
    date_added     = models.DateTimeField(_('date added'), auto_now_add=True)
    date_modified  = models.DateTimeField(_('date modified'), auto_now=True)

    # Misc fields
    page_type               = models.CharField(_('page type'), max_length=4, choices=TYPE_TYPES, default=TYPE_TYPES[0][0])

    title                   = models.CharField(_('title'), max_length=256)
    url                     = models.SlugField(_('url'), max_length=32)

    seo_title               = models.CharField(_('seo title'), max_length=256, null=True, blank=True)
    seo_keywords            = models.CharField(_('seo keywords'), max_length=256, null=True, blank=True)
    seo_description         = models.CharField(_('seo description'), max_length=256, null=True, blank=True)

    content                 = models.TextField(_('content'), blank=True, null=True)

    redirect_url            = models.CharField(_('redirect url'), max_length=256, null=True, blank=True)
    redirect_to_first_child = models.BooleanField(_('redirect to first child'), default=None)

    enabled                 = models.BooleanField(_('enabled'), default=None)

    class Meta:
        verbose_name = _('menu')
        verbose_name_plural = _('menus')
        unique_together = ("url", "parent")
        ordering = ('sortorder',)
        abstract = True

    def __unicode__(self):
        return "%s%s" % ("- " * self.level, self.title)

    def save(self, skip_tree_update=False, *args, **kwargs):
        if not skip_tree_update:
            if self.get_parent():
                parent = self.get_parent()
                self.parents_list = ';'.join([str(v) for v in parent.get_parents_ids_list() + [parent.pk]])
                # if not parent.has_childs:
                #     parent.has_childs = True
                #     parent.save(skip_tree_update=True)
            else:
                class Parent:
                    pass
                parent = Parent()
                parent.full_url = ''
                parent.level = -1
                parent.sortorder = ''
                parent.parents_list = ''
                self.parents_list = ''

            self.full_url = "%s%s/" % (parent.full_url, self.url)
            self.level = parent.level + 1

            self.sortorder = parent.sortorder + ('%' + '0%dd' % len(str(self.MAX_ITEMS))) % self.sort

            self.has_childs = False

            for child in self.get_childs():
                self.has_childs = True
                child._get_parent = self
                child.save()

        super(SiteMenu, self).save(*args, **kwargs)

    def get_parent(self):
        try:
            self._get_parent
        except:
            self._get_parent = self.parent
        return self._get_parent

    def get_parents_ids_list(self):
        return [int(v) for v in filter(None, self.parents_list.split(';'))]

    def get_childs(self):
        try:
            self._get_childs
        except:
            self._get_childs = self._default_manager.filter(parent=self).order_by('sort')
        return self._get_childs

    def create_tree(self, queryset):
        current_path = []
        top_nodes = []
        root_level = None
        for obj in queryset:
            node_level = obj.level
            if root_level is None:
                root_level = node_level
            if node_level < root_level:
                raise ValueError(_("cache_tree_children was passed nodes in the wrong order!"))
            obj._get_childs = []

            while len(current_path) > node_level - root_level:
                current_path.pop(-1)
            if node_level == root_level:
                top_nodes.append(obj)
            else:
                current_path[-1]._get_childs.append(obj)
            current_path.append(obj)
        return top_nodes

    def rebuild(self):
        for menu in self._default_manager.filter(parent=None):
            menu.save()

    def render(self, request, url_add):
        use_page = None
        for page in self.PAGES:
            if page[0] == self.page_type:
                use_page = page
        if not use_page:
            raise KeyError("Could not find page type '%s'" % self.page_type)

        return import_item(use_page[2])(request, self, url_add)

    def get_absolute_url(self):
        if self.redirect_url:
            return self.redirect_url
        if self.redirect_to_first_child:
            return self._default_manager.filter(parent=self)[0].get_absolute_url()
        if self.page_type == 'indx':
            return '/'
        return reverse('dispatcher', kwargs={'url': self.full_url})

    def is_active(self, full_path):
        if full_path == '/' and self.page_type == 'indx':
            return True
        return self.full_url in full_path

    def get_breadcrumbs(self):
        return self._default_manager.filter(pk__in=self.get_parents_ids_list() + [self.pk])

    def get_page_title(self):
        if self.page_type == 'indx':
            return None
        try:
            return self._page_title
        except:
            return self.title

if MENUCLASS == 'sitemenu.models.Menu':
    class Menu(SiteMenu):
        pass



#!/usr/bin/env python
from setuptools import setup

setup(name='django-sitemenu',
      version='0.0.2',
      description='Nested menu for django projects.',
      long_description='Nested menu for django projects. Without MPTT and with drag\'n\'drop sorting.'
                       'Easy customizable. Looks like simple model for using.',
      author='Vital Belikov',
      author_email='vital@qwe.lv',
      packages=['sitemenu', 'sitemenu.templatetags'],
      url='https://github.com/Brick85/sitemenu/',
      include_package_data=True,
      zip_safe=False,
      requires=['django(>=1.3)'],
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Natural Language :: English',
                   'Operating System :: Unix',
                   'Programming Language :: Python :: 2.7',
                   'Topic :: Utilities'],
      license='New BSD')
